function [handles, status] = convertImage(obj,format,handles)
% function [handles, status] = convertImage(obj,format,handles)
% Convert image to specified format: 'grayscale', 'truecolor', 'indexed' and 'uint8', 'uint16', 'uint32' class
%
% Parameters:
% format: description of the new image format
% - ''grayscale'' - grayscale image, 1 color channel
% - ''truecolor'' - truecolor image, 2 or 3 color channels (red, green, blue)
% - ''hsvcolor'' - hsv color image, 3 color channels (hue, saturation, value)
% - ''indexed'' - indexed colors, the color map is stored in @em imageData.img_info. @e Colormap
% - ''uint8'' - 8-bit unsinged integer, [0 - 255] levels
% - ''uint16'' - 16-bit unsinged integer, [0 - 65535] levels;
% - ''uint32'' - 32-bit unsinged integer, [0 - 4294967295] levels; @b Note! Not Really tested...
% handles: handles of im_browser.m
%
% Return values:
% handles: handles of im_browser.m
% status: @b 1 -success, @b 0 -fail

%| 
% @b Examples:
% @code handles = imageData.convertImage(handles, 'uint8');  // convert dataset to the uint8 class @endcode
% @code [handles, status] = convertImage(obj, handles, 'grayscale');   // Call within the class; convert dataset to the grayscale type @endcode

% Copyright (C) 30.10.2013, Ilya Belevich (ilya.belevich @ helsinki.fi)
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.

% Updates
% 


status = 0;
tic
wb = waitbar(0,['Converting image to ' format ' format'],'Name','Converting image','WindowStyle','modal');
if strcmp(format, 'grayscale')   % ->grayscale
    switch obj.img_info('ColorType')
        case 'grayscale'        % grayscale->
            delete(wb);
            return;
        case 'truecolor'    % truecolor->grayscale
            from = 'truecolor';
            if size(obj.img,3) > 3;
                button = questdlg(sprintf('!!! Attention !!!\n\nDirect conversion of the multichannel image to greyscale is not possible\nHowever it is possible to perform conversion using the LUT colors'),'Multiple color channels','Convert','Cancel','Cancel');
                if strcmp(button, 'Cancel'); return; end;
                if get(handles.lutCheckbox, 'value') == 0
                    errordlg('Please make sure that the LUT checkbox in the View settings panel is checked!','LUT is not selected');
                    return;
                end
                
                I = zeros([size(obj.img,1),size(obj.img,2),1,size(obj.img,4)], class(obj.img));
                selectedColorsLUT = obj.lutColors(obj.slicesColor,:);     % take LUT colors for the selected color channels
                max_int = double(intmax(class(obj.img)));
                for sliceId=1:obj.no_stacks
                    sliceImg = obj.img(:,:,obj.slicesColor,sliceId);    % get slice
                    R = zeros([size(sliceImg,1), size(sliceImg,2)], class(obj.img));
                    G = zeros([size(sliceImg,1), size(sliceImg,2)], class(obj.img));
                    B = zeros([size(sliceImg,1), size(sliceImg,2)], class(obj.img));
                    for colorId=1:numel(obj.slicesColor)
                        adjImg = imadjust(sliceImg(:,:,colorId),[obj.viewPort.min(obj.slicesColor(colorId))/max_int obj.viewPort.max(obj.slicesColor(colorId))/max_int],[0 1],obj.viewPort.gamma(obj.slicesColor(colorId)));
                        R = R + adjImg*selectedColorsLUT(colorId, 1);
                        G = G + adjImg*selectedColorsLUT(colorId, 2);
                        B = B + adjImg*selectedColorsLUT(colorId, 3);
                    end
                    imgRGB = cat(3,R,G,B);
                    I(:,:,1,sliceId) = rgb2gray(imgRGB);
                    waitbar(sliceId/obj.no_stacks,wb);
                end
                obj.img = I;
                set(handles.lutCheckbox, 'value', 0);
            else
                I = obj.img;
                if  size(I,3) == 1  % a single color channel
                    I(:,:,2,:) = zeros([size(I,1),size(I,2),size(I,4)],class(I));
                    I(:,:,3,:) = zeros([size(I,1),size(I,2),size(I,4)],class(I));
                elseif size(I,3) == 2   % two color channels
                    I(:,:,3,:) = zeros([size(I,1),size(I,2),size(I,4)],class(I));
                end
                obj.img = zeros(size(I,1),size(I,2),1,size(I,4),class(I));
                for i=1:obj.no_stacks
                    obj.img(:,:,1,i) = rgb2gray(I(:,:,:,i));
                    waitbar(i/obj.no_stacks,wb);
                end
            end
        case 'hsvcolor'    % hsvcolor->grayscale            
            delete(wb);
            errordlg('Please convert the image to RGB color!','Wrong image format!');
            return;
        case 'indexed'      % indexed->grayscale
            from = 'indexed';
            I = obj.img;
            obj.img = zeros(size(I,1),size(I,2),1,size(I,4),class(I));
            for i=1:obj.no_stacks
                obj.img(:,:,1,i) = ind2gray(I(:,:,:,i),obj.img_info('Colormap'));
                waitbar(i/obj.no_stacks,wb);
            end
            obj.img_info('Colormap') = '';
    end
    obj.img_info('ColorType') = 'grayscale';
elseif strcmp(format,'truecolor')   % ->truecolor
    switch obj.img_info('ColorType')
        case 'grayscale'    % grayscale->truecolor
            from = 'grayscale';
            I = obj.img;
            obj.img = zeros(size(I,1),size(I,2),3,size(I,4),class(I));
            obj.img(:,:,1,:) = I;
            obj.img(:,:,2,:) = I;
            obj.img(:,:,3,:) = I;
            waitbar(.85,wb);
        case 'truecolor'    % truecolor->truecolor
            delete(wb);
            return;
        case 'hsvcolor'    % hsvcolor->truecolor
            from = 'hsvcolor';
            I = obj.img;
            obj.img = zeros(size(I,1),size(I,2),3,size(I,4),'uint8');
            for i=1:size(I,4)
                %obj.img(:,:,:,i) = uint8(hsv2rgb( double(I(:,:,:,i))/255));
                obj.img(:,:,:,i) = uint8(hsv2rgb(double(I(:,:,:,i))/255)*255);
            end
            waitbar(.85,wb);
        case 'indexed'      % indexed->truecolor
            from = 'indexed';
            I = obj.img;
            obj.img = zeros(size(I,1),size(I,2),3,size(I,4),class(I));
            max_int = double(intmax(class(I)));
            for i=1:obj.no_stacks
                obj.img(:,:,:,i) = ind2rgb(I(:,:,:,i),obj.img_info('Colormap'))*max_int;
                waitbar(i/obj.no_stacks,wb);
            end
            obj.img_info('Colormap') = '';
    end
    obj.img_info('ColorType') = 'truecolor';
elseif strcmp(format,'hsvcolor')   % ->hsvcolor
    switch obj.img_info('ColorType')
        case 'grayscale'    % grayscale->hsvcolor
            delete(wb);
            errordlg('Please convert the image to RGB color!','Wrong image format!');
            return;
        case 'truecolor'    % truecolor->hsvcolor
            from = 'truecolor';
            if size(obj.img,3) ~= 3;
                delete(wb);
                errordlg('Please convert the image to RGB color!','Wrong image format!');
                return;
            end
            I = obj.img;
            obj.img = zeros(size(I,1),size(I,2),3,size(I,4),'uint8');
            for i=1:size(I,4)
                obj.img(:,:,:,i) = uint8(rgb2hsv(I(:,:,:,i))*255);
            end
            waitbar(.85,wb);
        case 'hsvcolor'
            delete(wb);
            return;            
        case 'indexed'      % indexed->hsvcolor
            delete(wb);
            errordlg('Please convert the image to RGB color!','Wrong image format!');
            return;
    end
    obj.img_info('ColorType') = 'hsvcolor';    
elseif strcmp(format,'indexed')   % ->indexed
    if strcmp(obj.img_info('ColorType'),'indexed') % indexed->indexed
        delete(wb);
        return;
    end
    if strcmp(obj.img_info('ColorType'),'hsvcolor') % hsvcolor->indexed
            delete(wb);
            errordlg('Please convert the image to RGB color!','Wrong image format!');
            return;
    end
    %answer = inputdlg(sprintf('Please enter number of graylevels\n [1-65535]'),'Convert to indexed image',1,{'255'});
    answer = mib_inputdlg(NaN, sprintf('Please enter number of graylevels\n [1-65535]'),'Convert to indexed image','255');
    if isempty(answer);  delete(wb); return; end;
    levels = round(str2double(cell2mat(answer)));
    if levels >= 1 && levels <=255
        class_id = 'uint8';
    elseif levels > 255 && levels <= 65535
        class_id = 'uint16';
    else
        delete(wb);
        msgbox('Wrong number of gray levels','Error','error');
        return;
    end
    switch obj.img_info('ColorType')
        case 'grayscale'    % grayscale->indexed
            from = 'grayscale';
            I = obj.img;
            obj.img = zeros(size(I,1),size(I,2),1,size(I,4),class_id);
            for i=1:obj.no_stacks
                [obj.img(:,:,1,i), obj.img_info('Colormap')] =  gray2ind(I(:,:,1,i),levels);
                waitbar(i/obj.no_stacks,wb);
            end
        case 'truecolor'    % truecolor->indexed
            from = 'truecolor';
            if size(obj.img,3) > 3;
                button = questdlg(sprintf('!!! Attention !!!\n\nDirect conversion of the multichannel image to greyscale is not possible\nHowever it is possible to perform conversion using the LUT colors'),'Multiple color channels','Convert','Cancel','Cancel');
                if strcmp(button, 'Cancel'); return; end;
                if get(handles.lutCheckbox, 'value') == 0
                    errordlg('Please make sure that the LUT checkbox in the View settings panel is checked!','LUT is not selected');
                    return;
                end
                
                I = zeros([size(obj.img,1),size(obj.img,2),1,size(obj.img,4)], class_id);
                selectedColorsLUT = obj.lutColors(obj.slicesColor,:);     % take LUT colors for the selected color channels
                max_int = double(intmax(class(obj.img)));
                for sliceId=1:obj.no_stacks
                    sliceImg = obj.img(:,:,obj.slicesColor,sliceId);    % get slice
                    R = zeros([size(sliceImg,1), size(sliceImg,2)], class(obj.img));
                    G = zeros([size(sliceImg,1), size(sliceImg,2)], class(obj.img));
                    B = zeros([size(sliceImg,1), size(sliceImg,2)], class(obj.img));
                    for colorId=1:numel(obj.slicesColor)
                        adjImg = imadjust(sliceImg(:,:,colorId),[obj.viewPort.min(obj.slicesColor(colorId))/max_int obj.viewPort.max(obj.slicesColor(colorId))/max_int],[0 1],obj.viewPort.gamma(obj.slicesColor(colorId)));
                        R = R + adjImg*selectedColorsLUT(colorId, 1);
                        G = G + adjImg*selectedColorsLUT(colorId, 2);
                        B = B + adjImg*selectedColorsLUT(colorId, 3);
                    end
                    imgRGB = cat(3,R,G,B);
                    [I(:,:,1,sliceId), obj.img_info('Colormap')] = rgb2ind(imgRGB, levels);
                    waitbar(sliceId/obj.no_stacks,wb);
                end
                obj.img = I;
                set(handles.lutCheckbox, 'value', 0);
            else
                I = obj.img;
                obj.img = zeros(size(I,1),size(I,2),1,size(I,4),class_id);
                for i=1:obj.no_stacks
                    [obj.img(:,:,1,i), obj.img_info('Colormap')] =  rgb2ind(I(:,:,:,i),levels);
                    waitbar(i/obj.no_stacks,wb);
                end
            end
    end
    obj.img_info('ColorType') = 'indexed';
elseif strcmp(format,'uint8')   % -> uint8
    if strcmp(obj.img_info('ColorType'),'indexed')
        msgbox('Convert to RGB or Grayscale first','Error','error');
        delete(wb);
        return;
    end
    switch class(obj.img)
        case 'uint8'
            delete(wb);
            return;
        case 'uint16'       % uint16->uint8 
            from = class(obj.img);
            obj.img = uint8(obj.img / (double(intmax('uint16'))/double(intmax('uint8'))));
        case 'uint32'       % uint32->uint8
            from = class(obj.img);
            obj.img = uint8(obj.img / (double(intmax('uint32'))/double(intmax('uint8'))));
    end
elseif strcmp(format,'uint16')   % -> uint16
    if strcmp(obj.img_info('ColorType'),'indexed')
        msgbox('Convert to RGB or Grayscale first','Error','error');
        delete(wb);
        return;
    end
    switch class(obj.img)
        case 'uint16'
            delete(wb);
            return;
        case 'uint8'       % uint8->uint16 
            from = class(obj.img);
            obj.img = uint16(obj.img)*(double(intmax('uint16'))/double(intmax('uint8')));
        case 'uint32'    % uint32->uint16
            from = class(obj.img);
            obj.img = uint32(obj.img)*(double(intmax('uint32'))/double(intmax('uint8')));
    end
elseif strcmp(format,'uint32')   % -> uint32
    if strcmp(obj.img_info('ColorType'),'indexed')
        msgbox('Convert to RGB or Grayscale first','Error','error');
        delete(wb);
        return;
    end
    switch class(obj.img)
        case 'uint32'
            delete(wb);
            return;
        case 'uint8'       % uint8->uint32 
            msgbox('Not implemented','Error','error');
            delete(wb);
            return;
            %from = class(obj.img);
            %obj.img = uint32(obj.img)*(double(intmax('uint32'))/double(intmax('uint8')));
        case 'uint16'      % uint16->uint32
            msgbox('Not implemented','Error','error');
            delete(wb);
            return;
            %from = class(obj.img);
            %obj.img = uint32(obj.img)*(double(intmax('uint32'))/double(intmax('uint16')));
    end
end

obj.colors = size(obj.img,3);
obj.slices(3,:) = [1 obj.colors];    % color slices to show
obj.slicesColor = 1:obj.colors;


% update display parameters
obj.updateDisplayParameters();
handles = updateGuiWidgets(handles);

log_text = ['Converted to from ' from ' to ' format];
obj.updateImgInfo(log_text);
delete(wb);
status = 1;
toc
end
