function dataset = getData4D(obj, type, time, orient, col_channel, options, custom_img)
% function dataset = getData4D(obj, type, time, orient, col_channel, options, custom_img)
% Get the 4D dataset: height:width:colors:depth
%
% Parameters:
% type: type of the slice to retrieve, 'image', 'model','mask', 'selection', 'custom' ('custom' indicates to use custom_img as the dataset), 'everything'('model','mask' and 'selection' for imageData.model_type==''uint6'' only)
% time: [@em optional], an index of the time point to show, when @em NaN gets the dataset for the current time point
% orient: [@em optional], can be @em NaN 
% @li when @b 0 (@b default) returns the dataset transposed to the current orientation (obj.orientation)
% @li when @b 1 returns transposed dataset to the zx configuration: [y,x,c,z] -> [x,z,c,y]
% @li when @b 2 returns transposed dataset to the zy configuration: [y,x,c,z] -> [y,z,c,y]
% @li when @b 3 not used
% @li when @b 4 returns original dataset to the yx configuration: [y,x,c,z]
% @li when @b 5 not used
% col_channel: [@em optional], can be @e NaN
% @li when @b type is 'image', col_channel is a vector with color numbers to take, when @b NaN [@e default] take the colors
% selected in the imageData.slicesColor variable, when @b 0 - take all colors of the dataset.
% @li when @b type is 'model' col_channel may be @em NaN - to take all materials of the model or an integer to take specific material. In the later case the selected material in @b slice will have index = 1.
% custom_img: [@em optional], can be @e NaN; the function return a slice not from the imageData class but from this custom_img, requires to
% specify the 'custom' @b type. 'custom_img' should be a 3D dataset.
% options: [@em optional], a structure with extra parameters
% @li .blockModeSwitch -> override the imageData.blockModeSwitch (@b 0 - return full dataset, @b 1 - return only the shown part)
% @li .y -> [@em optional], [ymin, ymax] of the part of the dataset to take
% @li .x -> [@em optional], [xmin, xmax] of the part of the dataset to take
% @li .z -> [@em optional], [zmin, zmax] of the part of the dataset to take
%
% Return values:
% dataset: 4D image. For the 'image' type: [1:height, 1:width, 1:colors, 1:depth]; for all other types: [1:height, 1:width, 1:depth]

%| 
% @b Examples:
% @code dataset = handles.Img{handles.Id}.I.getData4D('image');      // get the 4D dataset for the current time point, in the shown orientation  @endcode
% @code dataset = handles.Img{handles.Id}.I..getData4D('image', 5, 4, 2); // get the 4D dataset for the 5-th time point in the XY orientation @endcode
% @code dataset = handles.Img{handles.Id}.I..getData4D('selection', 5, 1, 2, custom_img); // get the 5-th timepoint in the the XZ-orientation, color channel=2 @endcode
% @code dataset = obj.getData4D(img, 'image', [], 4);      // Call within the class; get the 4D dataset for the current time point in the XY orientation  @endcode
% @attention @b sensitive to the @code imageData.blockModeSwitch @endcode
% @attention @b not @b sensitive to the shown ROI
% @attention for normal calls it is recommended to use ib_getDataset.m wrapper function, as @code
% dataset = ib_getDataset('image', handles); @endcode

% Copyright (C) 03.09.2015, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

if nargin < 7; custom_img = NaN; end;
if nargin < 6; options = struct();  end;
if nargin < 5; col_channel = NaN;   end;
if nargin < 4; orient = NaN; end;
if nargin < 3; time = NaN; end;

if isnan(time); time = 1; end;
if orient == 0 || isnan(orient); orient=obj.orientation; end;

if strcmp(type,'image')
    if isnan(col_channel); col_channel=obj.slicesColor; end;
    if col_channel(1) == 0;  col_channel = 1:size(obj.img,3); end;
end

options.t = [time time];   % define the time point
dataset = obj.getData5D(type, orient, col_channel, options, custom_img);
end