function mark2selection(obj, layer_id, type, str_type, xy_vec)
% function mark2selection(obj, layer_id, type, str_type, xy_vec)
% Add a marker/pointer to the selection layer
%
% Parameters:
% layer_id: Index of the z-slice, for example (1)
% type: a type of the marker:
% - ''retract'' - a minus sign,
% - ''growth'' - a plus sign
% - ''sliding'' - the x sign
% - ''cross'' - a cross sign
% - ''large cross'' - a larger cross sign
% str_type: an additional identifier:
% - ''tube'' - draw a line on the left side of the marker
% - ''sheet'' - draw a line of the right side of the marker
% - ''unknown'' - keep the marker without line
% xy_vec: - coordinates of the markers, [1:number of points, [x, y]]
%
% Return values:

%| 
% @b Examples:
% @code xy_vec(1, :) = [50 75];  @endcode
% @code imageData.mark2selection(1, 'cross', '', xy_vec); // Draw a small cross at X=50 Y=75 @endcode
% @code mark2selection(obj, 1, 'cross', '', xy_vec); // Call within the class @endcode
% @attention Implemented only for the XY-orientation

% Copyright (C) 21.11.2013, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

type = [str_type type];
selection_layer = obj.selection(:,:,layer_id);
for ind = 1: size(xy_vec,1)
    xy = round(xy_vec(ind,:));
    if xy(1)-3 < 1; xy(1) = 3+1; end;
    if xy(2)-3 < 1; xy(2) = 3+1; end;
    if xy(1)+3 > size(obj.selection,2); xy(1) = size(obj.selection,2)-3; end;
    if xy(2)+3 > size(obj.selection,1); xy(2) = size(obj.selection,1)-3; end;
    
    switch type
        case 'tuberetract'
            sel_mask = ...
                [1 0 0 0 0;...
                1 0 0 0 0;...
                1 0 1 1 1;...
                1 0 0 0 0;...
                1 0 0 0 0];
        case 'sheetretract'
            sel_mask = ...
                [0 0 0 0 1;...
                0 0 0 0 1;...
                1 1 1 0 1;...
                0 0 0 0 1;...
                0 0 0 0 1];
        case 'unknownretract'
            sel_mask = ...
                [0 0 0 0 0;...
                0 0 0 0 0;...
                0 1 1 1 0;...
                0 0 0 0 0;...
                0 0 0 0 0];
        case 'tubegrowth'
            sel_mask = ...
                [1 0 0 0 0;...
                1 0 0 1 0;...
                1 0 1 1 1;...
                1 0 0 1 0;...
                1 0 0 0 0];
        case 'sheetgrowth'
            sel_mask = ...
                [0 0 0 0 1;...
                0 1 0 0 1;...
                1 1 1 0 1;...
                0 1 0 0 1;...
                0 0 0 0 1];
        case 'unknowngrowth'
            sel_mask = ...
                [0 0 0 0 0;...
                0 0 1 0 0;...
                0 1 1 1 0;...
                0 0 1 0 0;...
                0 0 0 0 0];
        case 'tubesliding'
            sel_mask = ...
                [1 0 0 0 0;...
                1 0 1 0 1;...
                1 0 0 1 0;...
                1 0 1 0 1;...
                1 0 0 0 0];
        case 'sheetsliding'
            sel_mask = ...
                [0 0 0 0 1;...
                1 0 1 0 1;...
                0 1 0 0 1;...
                1 0 1 0 1;...
                0 0 0 0 1];
        case 'unknownsliding'
            sel_mask = ...
                [0 0 0 0 0;...
                0 1 0 1 0;...
                0 0 1 0 0;...
                0 1 0 1 0;...
                0 0 0 0 0];
        case 'cross'
            sel_mask = ...
                [0 0 0 0 0;...
                0 0 1 0 0;...
                0 1 1 1 0;...
                0 0 1 0 0;...
                0 0 0 0 0];
        case 'largecross'
            sel_mask = ...
                [0 0 1 0 0;...
                0 0 1 0 0;...
                1 1 1 1 1;...
                0 0 1 0 0;...
                0 0 1 0 0];
    end
    selection_layer(xy(2)-2:xy(2)+2, xy(1)-2:xy(1)+2) = ...
        selection_layer(xy(2)-2:xy(2)+2, xy(1)-2:xy(1)+2) | sel_mask;
    
end
obj.selection(:,:,layer_id) = selection_layer;
end