function custom_img = setSliceToShow(obj, type, slice, slice_no, orient, col_channel, custom_img,options)
% function custom_img = setSliceToShow(obj, type, slice, slice_no, orient, col_channel, custom_img)
% Update the croped to the viewing window 2D slice of the dataset
%
% Parameters:
% type: type of the image to update, 'image', 'model', 'mask', 'selection', or 'everything' ('model','mask' and 'selection' for imageData.model_type==''uint6'' only)
% slice: 2D image, for 'image' as [1:height,1:width,1:color]; for other layers as [1:height,1:width]
% slice_no: [@em optional] an index of the slice to update, when @em empty or @em NaN updates the current slice
% orient: [@em optional] update a slice in the desired orientation
% - @b 1 - xz
% - @b 2 - yz
% - @b 4 - xy
% - @b NaN - use the current orientation
% col_channel: [@em optional], can be @em NaN
% @li when @b type is 'image', col_channel is a vector with color numbers to update, when @b NaN [@e default] update set the
% colors selected in the imageData.slicesColor variable; when @b 0 - update all colors of the dataset.
% @li when @b type is 'model' col_channel may be @em NaN - to update all materials of the model or an integer to update specific material.
% custom_img: [@em optional], can be @e NaN; update a slice not from the imageData class but from this custom_img (3D stack), requires the 'custom' @b type
% options: [@em optional], a structure with extra parameters
% @li .y -> [@em optional], [ymin, ymax] of the part of the slice to update
% @li .x -> [@em optional], [xmin, xmax] of the part of the slice to update
%
% Return values:
% custom_img: a modified 3D stack, for the 'custom' type; or a
% status of the function run: @b - success, @b - fail.

%| 
% @b Examples:
% @code imageData.setSliceToShow('image', slice, 5);      // assign slice to the 5-th slice of the current stack orientation  @endcode
% @code imageData.setSliceToShow('image', slice, 5, 4, 2); // assign slice to the 5-th slice of the XY-orientation, color channel=2 @endcode
% @code custom_img = imageData.setSliceToShow('custom', slice, 5, 1, 2, custom_img); // assign slice to the 5-th slice of the YZ-orientation, color channel=2 to the custom_image @endcode
% @code setSliceToShow(obj, 'image', slice, NaN, NaN, 2); // Call within the class; assign slice to the current slice of the current orientation to color channel=2 @endcode
% @attention @b not @b sensitive to the @code imageData.blockModeSwitch @endcode
% @attention @b not @b sensitive to the shown ROI
% @attention consider to use ib_setSlice.m wrapper function

% Copyright (C) 20.06.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


if nargin < 8; options = struct();   end;
if nargin < 7; custom_img = NaN; end;
if nargin < 6; col_channel = NaN; end;
if nargin < 5; orient = NaN; end;
if nargin < 4; slice_no=NaN; end;

if isnan(slice_no); slice_no=obj.slices(obj.orientation, 1); end;
if isnan(orient); orient=obj.orientation; end;
if strcmp(type,'image')
    if isnan(col_channel); col_channel=obj.slicesColor; end;
    if col_channel(1) == 0; col_channel = 1:size(obj.img,3); end;
end

optionsGetDims.blockModeSwitch = 0;
[height,width] = obj.getDatasetDimensions('image', NaN, NaN, optionsGetDims);
if isfield(options, 'y');
    Xlim(1) = max([options.x(1) 1]);
    Xlim(2) = min([options.x(2) width]);
    Ylim(1) = max([options.y(1) 1]);
    Ylim(2) = min([options.y(2) height]);
else
    Xlim = ceil(obj.axesX);
    Ylim = ceil(obj.axesY);
end

if ~strcmp(obj.model_type, 'uint6')
    if strcmp(type,'image')
        if orient==1     % xz
            obj.img(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),col_channel,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = permute(slice, [1 3 2]);
        elseif orient==2 % yz
            obj.img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,col_channel,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = permute(slice, [1 3 2]);
        elseif orient==4 % yx
            obj.img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),col_channel,slice_no) = slice;
        end
    elseif strcmp(type,'model')
        if isnan(col_channel)     % do for all objects
            if orient==1     % xz
                obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
            elseif orient==2 % yz
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
            elseif orient==4 % yx
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = slice;
            end
        else
            existingSlice = obj.getSlice('model', slice_no, orient);
            existingSlice(existingSlice==col_channel) = 0;
            existingSlice(slice==1) = col_channel;
            if orient==1     % xz
                obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = existingSlice;
            elseif orient==2 % yz
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = existingSlice;
            elseif orient==4 % yx
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = existingSlice;
            end
        end
    elseif strcmp(type,'mask')
        if obj.maskExist == 0  % create maskImg
            obj.maskImg = zeros(size(obj.img,1),size(obj.img,2),size(obj.img,4),'uint8');
        end
        if orient==1     % xz
            obj.maskImg(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==2 % yz
            obj.maskImg(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==4 % yx
            obj.maskImg(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = slice;
        end
        obj.maskExist = 1;
    elseif strcmp(type,'selection')
        if orient==1     % xz
            obj.selection(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==2 % yz
            obj.selection(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==4 % yx
            obj.selection(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = slice;
        end
    elseif strcmp(type,'custom')
        if orient==1     % xz
            custom_img(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==2 % yz
            custom_img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==4 % yx
            custom_img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = slice;
        end
    end
else            % ************ uint6 model type
    if islogical(slice(1)); slice = uint8(slice); end;
    if strcmp(type,'image')
        if orient==1     % xz
            obj.img(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),col_channel,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = permute(slice, [1 3 2]);
        elseif orient==2 % yz
            obj.img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,col_channel,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = permute(slice, [1 3 2]);
        elseif orient==4 % yx
            obj.img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),col_channel,slice_no) = slice;
        end
    elseif strcmp(type,'model')
        if isnan(col_channel)     % do for all objects
            if orient==1     % xz
                existingSlice = squeeze(obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])));
                existingSlice = bitor(slice, bitand(existingSlice, 192));
                obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = existingSlice;
            elseif orient==2 % yz
                existingSlice = squeeze(obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])));
                existingSlice = bitor(slice, bitand(existingSlice, 192));
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = existingSlice;
            elseif orient==4 % yx
                existingSlice = obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no);
                existingSlice = bitor(slice, bitand(existingSlice, 192));
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = existingSlice;
            end
        else
            existingSlice = obj.getSliceToShow('everything', slice_no, orient);
            % clear the selected material number
            existingSlice(bitand(slice, 1)==1) = bitand(existingSlice(bitand(slice, 1)==1), 192);  % 192 = 11000000
            existingSlice(slice==1) = bitor(existingSlice(slice==1), col_channel);
            if orient==1     % xz
                obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = existingSlice;
            elseif orient==2 % yz
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = existingSlice;
            elseif orient==4 % yx
                obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = existingSlice;
            end
        end
    elseif strcmp(type,'mask')
        if orient==1     % xz
            modelSlice = squeeze(bitset(obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])),7,0));
            %modelSlice(slice==1) = bitset(modelSlice(slice==1), 7, 1);
            obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = bitor(modelSlice, slice*64);
        elseif orient==2 % yz
            modelSlice = squeeze(bitset(obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])),7,0));
            %modelSlice(slice==1) = bitset(modelSlice(slice==1), 7, 1);
            obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = bitor(modelSlice, slice*64);
        elseif orient==4 % yx
            modelSlice = bitset(obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no),7,0);
            %modelSlice(slice==1) = bitset(modelSlice(slice==1), 7, 1);
            obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = bitor(modelSlice, slice*64);
        end
        obj.maskExist = 1;
    elseif strcmp(type,'selection')
        if orient==1     % xz
            modelSlice = squeeze(obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])));
            modelSlice = bitset(modelSlice, 8, 0);  % clear selection layer in the obtained image
            %modelSlice(slice==1) = bitset(modelSlice(slice==1), 8, 1);
            obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = bitor(modelSlice, slice*128);
        elseif orient==2 % yz
            modelSlice = squeeze(obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])));
            modelSlice = bitset(modelSlice, 8, 0); % clear selection layer in the obtained image
            %modelSlice(slice==1) = bitset(modelSlice(slice==1), 8, 1);
            obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = bitor(modelSlice, slice*128);
        elseif orient==4 % yx
            modelSlice = obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no);
            modelSlice = bitset(modelSlice, 8, 0); % clear selection layer in the obtained image
            %modelSlice(slice==1) = bitset(modelSlice(slice==1), 8, 1);
            obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = bitor(modelSlice, slice*128);
        end
    elseif strcmp(type, 'everything')
        if orient==1     % xz
            obj.model(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==2 % yz
            obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==4 % yx
            obj.model(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = slice;
        end
    elseif strcmp(type,'custom')
        if orient==1     % xz
            custom_img(slice_no,max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==2 % yz
            custom_img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),slice_no,max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks])) = slice;
        elseif orient==4 % yx
            custom_img(max([Ylim(1) 1]):min([Ylim(2) obj.height]),max([Xlim(1) 1]):min([Xlim(2) obj.width]),slice_no) = slice;
        end
    end
end
if isnan(custom_img(1)); custom_img = 1; end;
end