function handles = ib_anisotropicDiffusion(handles, filter_type)
% function handles = ib_anisotropicDiffusion(handles, filter_type)
% Filter image with Anisotropic diffusion filters
% 
% Parameters:
% handles: structure with handles of im_browser.m
% filter_type: type of desired filter:
%  - ''diplib'', use diplib library to do the filtering (http://www.diplib.org/)
%  - ''anisodiff'', use anisodiff function by Peter Kovesi (http://www.csse.uwa.edu.au/~pk/Research/MatlabFns/#anisodiff)
%  - ''coherence_filter'', use Image Edge Enhancing Coherence Filter by Dirk-Jan Kroon and Pascal Getreuer (http://www.mathworks.com/matlabcentral/fileexchange/25449-image-edge-enhancing-coherence-filter-toolbox)
%
% Return values:
% handles: structure with handles of im_browser.m

% Copyright (C) 21.11.2013, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

switch8bit = get(handles.menuImage8bit,'Checked');
% if strcmp(switch8bit,'off');
%     msgbox('Please convert the dataset to 8bit format!','Error','error','modal');
%     return;
% end
switchIndexed = get(handles.menuImageIndexed,'Checked');
if strcmp(switchIndexed,'on');
    msgbox('Indexed images are not supported!','Error','error','modal');
    return;
end
if strcmp(filter_type,'diplib') || strcmp(filter_type, 'anisodiff')    % do diplib or anisodiff filtering
    colorChannel = get(handles.ColChannelCombo,'Value') - 1;
    if colorChannel == 0
        button = questdlg(sprintf('Attention!\nEach color channel will be filtered individually!\n\nIf you want to filter only one color channel please select it in the\nSelection Panel->Color channel'),'Warning','Proceed','Cancel','Proceed');
        if strcmp(button,'Cancel'); return;end;
    end
    tic
    val = get(handles.imageFilterPopup,'Value');
    opt = get(handles.imageFilterPopup,'String');
    if ischar(opt)
        sel_filter = opt;
    else
        sel_filter = opt{val};
    end
    options.Iter = str2double(get(handles.imfiltPar1Edit,'String'));
    options.KSigma = str2double(get(handles.imfiltPar2Edit,'String'));
    options.Lambda = str2double(get(handles.imfiltPar3Edit,'String'));
    options.Orientation = handles.Img{handles.Id}.I.orientation;
    options.Color = colorChannel;
    options.Favours = get(handles.andiffPopupFavours, 'value'); % 1: favours high contrast edges over low contrast ones; 2: favours wide regions over smaller ones. For 'anisodiff' only
    switch sel_filter
        case 'Perona Malik anisotropic diffusion'; 
            options.Filter = 'anisodiff'; 
            options.KSigma = options.KSigma/100*double(intmax(class(handles.Img{handles.Id}.I.img)));   % convert K from %% to numbers
        case 'Diplib: Perona Malik anisotropic diffusion'; options.Filter = 'pmd';
        case 'Diplib: Robust Anisotropic Diffusion'; options.Filter = 'aniso';
        case 'Diplib: Mean Curvature Diffusion'; options.Filter = 'mcd';
        case 'Diplib: Corner Preserving Diffusion'; options.Filter = 'cpf';
        case 'Diplib: Kuwahara filter for edge-preserving smoothing'; options.Filter = 'kuwahara';
    end
    options.start_no = handles.Img{handles.Id}.I.getCurrentSliceNumber();
    options.end_no = options.start_no;
    switch3d = 0;
    if get(handles.imageFilterAllCheck,'Value')
        options.start_no = 1;
        options.end_no = size(handles.Img{handles.Id}.I.img, handles.Img{handles.Id}.I.orientation);
        switch3d = 1;
    end
    ib_do_backup(handles, 'image', switch3d);
    img = ib_getDataset('image', handles, NaN, colorChannel);
    [img, status] = do_andiff_filtering(img, options);
    if status == 0; return; end;
    ib_setDataset('image', img, handles, NaN, colorChannel);
    log_text = [options.Filter ': Iter/Shape=' num2str(options.Iter) ';KSigma=' num2str(options.KSigma) ';lambda=' num2str(options.Lambda) ';Orient=' num2str(options.Orientation) 'Color=' num2str(options.Color)];
    handles.Img{handles.Id}.I.updateImgInfo(log_text);
elseif strcmp(filter_type,'coherence_filter')   % do coherence filtering
    colorChannel = get(handles.ColChannelCombo,'Value') - 1;
    if colorChannel == 0
        msgbox('Please select the color channel!','Error!','error','modal');
        return;
    end
    
    [options, dim] = anDiffOptionsDlg('options', NaN);
    if ~isstruct(options)
        return;
    end
    
    wb = waitbar(0,'Please wait...','Name','Anisotropic Diffusion filter...','WindowStyle','modal');
    Options = options;
    im_info = ['AnDiffFilter: Scheme=' Options.Scheme ...
        ',eigenmode=' num2str(Options.eigenmode) ',T=' num2str(Options.T)...
        ',dt=' num2str(Options.dt) ',sigma=' num2str(Options.sigma)...
        ',rho=' num2str(Options.rho) ',C=' num2str(Options.C)...
        ',m=' num2str(Options.m) ',alpha=' num2str(Options.alpha)...
        ',lambda_e=' num2str(Options.lambda_e) ',lambda_c=' num2str(Options.lambda_c)...
        ',lambda_h=' num2str(Options.lambda_h)];
    tic;
    if strcmp(dim,'2d')   % 2D mode
        if get(handles.imageFilterAllCheck,'Value')
            ib_do_backup(handles, 'image', 1);
            img = ib_getDataset('image', handles, NaN);        
        else
            ib_do_backup(handles, 'image', 0);
            img = ib_getSlice('image', handles);        
        end
        for roi=1:numel(img)
            img2 = img{roi};
            parfor index=1:size(img2, 4)
                img2(:,:,colorChannel,index) = uint8(CoherenceFilter(img2(:,:,colorChannel,index),Options));
            end
        end
        img{roi} = img2;
        if get(handles.imageFilterAllCheck,'Value')
            ib_setDataset('image', img, handles, NaN);
        else
            ib_setSlice('image', img, handles);
        end
        clear img img2;
    elseif strcmp(dim,'3d')   % 3D mode
        waitbar(.1,wb);
        ib_do_backup(handles, 'image', 1);
        img = ib_getDataset('image', handles, 4, colorChannel);
        for roi=1:numel(img)
            fImg = uint8(CoherenceFilter(squeeze(img{roi}),Options));
            img{roi} = reshape(fImg,size(fImg,1),size(fImg,2),1,size(fImg,3));
        end
        waitbar(.9,wb);
        ib_setDataset('image', img, handles, 4, colorChannel);
        waitbar(.95,wb);
    end
    handles.Img{handles.Id}.I.updateImgInfo(im_info);
    delete(wb);
elseif strcmp(filter_type,'coherence_filter_test')
    if handles.Img{handles.Id}.I.orientation ~= 4;
        msgbox('Please rotate the dataset to the XY orientation!','Error!','error','modal');
        return;
    end
    colorChannel = get(handles.ColChannelCombo,'Value') - 1;
    if colorChannel == 0
        msgbox('Please select the color channel!','Error!','error','modal');
        return;
    end
    dx = size(handles.Img{handles.Id}.I.img,2);
    dy = size(handles.Img{handles.Id}.I.img,1);
    dz = size(handles.Img{handles.Id}.I.img,4);
    w = round(handles.Img{handles.Id}.I.axesX);
    h = round(handles.Img{handles.Id}.I.axesY);
    if w(1)<0; w(1)=1; end;
    if w(2)>dx; w(2)=dx; end;
    if h(1)<0; h(1)=1; end;
    if h(2)>dy; h(2)=dy; end;
    
    prompt = {sprintf('Select small region for test run\nImage size(w:h:z) %ix%ix%i pixels\n\nNew width (w1:w2):',dx,dx,dz),'New height (h1:h2):','New stack number (z1:z2)'};
    title = 'Define Region';
    lines = 1;
    slice_no = handles.Img{handles.Id}.I.getCurrentSliceNumber();
    if slice_no + 2 > dz
        z_max = dz;
        z_min = dz-4;
    elseif slice_no - 2 < 1
        z_min = 1;
        z_max = 5;
    else
        z_min = slice_no-2;
        z_max = slice_no+2;
    end
    if z_max > size(handles.Img{handles.Id}.I.img,4); z_max = size(handles.Img{handles.Id}.I.img,4); end;
    if z_min < 1; z_min = 1; end;
    
    def = {[num2str(w(1)) ':' num2str(w(2))],[num2str(h(1)) ':' num2str(h(2))],[num2str(z_min) ':' num2str(z_max)]};
    answer = inputdlg(prompt,title,lines,def,'on');
    if size(answer) == 0; return; end;
    tic
    options.x(1) = str2double(answer{1}(1:strfind(answer{1},':')-1));
    options.x(2) = str2double(answer{1}(strfind(answer{1},':')+1:end));
    options.y(1) = str2double(answer{2}(1:strfind(answer{2},':')-1));
    options.y(2) = str2double(answer{2}(strfind(answer{2},':')+1:end));
    options.z(1) = str2double(answer{3}(1:strfind(answer{3},':')-1));
    options.z(2) = str2double(answer{3}(strfind(answer{3},':')+1:end));
    img = handles.Img{handles.Id}.I.getData4D('image', NaN, 4, NaN, options);
    anDiffOptionsDlg('test', img);
end
toc;
end