function handles = ib_contrastCLAHE(handles, current_sw)
% function handles = ib_contrastCLAHE(handles, current_sw)
% Do CLAHE Contrast-limited adaptive histogram equalization for the XY plane of the dataset for the currently shown or
% all slices
%
% Parameters:
% handles: structure with handles of im_browser.m
% current_sw: current slice switch
% - @b 1 - filter only the curretly shown slice
% - @b 0 - filter all slices
%
% Return values:
% handles: structure with handles of im_browser.m

% Copyright (C) 21.11.2013, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

colCh = get(handles.ColChannelCombo,'Value')-1;
prompt = {sprintf('You are going to change image contrast by Contrast-limited adaptive histogram equalization for color channel:%d\nYou can always undo it with Ctrl-Z\n\nEnter Number of Tiles:', colCh),...
    'Enter Clip Limit in the range [0 1] that specifies a contrast enhancement limit. Higher numbers result in more contrast:',...
    'Enter NBins, a positive integer scalar specifying the number of bins for the histogram used in building a contrast enhancing transformation. Higher values result in greater dynamic range at the cost of slower processing speed',...
    'Enter Distribution [uniform, rayleigh, exponential]:',...
    'Enter Alpha, a nonnegative real scalar specifying a distribution parameter, not for uniform distribution:'};
dlg_title = 'Enter CLAHE parameters';

def = {[num2str(handles.CLAHE.NumTiles(1)) ',' num2str(handles.CLAHE.NumTiles(2))],...
    num2str(handles.CLAHE.ClipLimit),num2str(handles.CLAHE.NBins),...
    handles.CLAHE.Distribution,num2str(handles.CLAHE.Alpha)};

answer = inputdlg(prompt,dlg_title,1,def);
if isempty(answer); return; end;

wb = waitbar(0,'Adjusting contrast with CLAHE...','Name','CLAHE','WindowStyle','modal');
str2 = cell2mat(answer(1));
commas = strfind(str2,',');
handles.CLAHE.NumTiles(1) = str2double(str2(1:commas(end)-1));
handles.CLAHE.NumTiles(2) = str2double(str2(commas(end)+1:end));
handles.CLAHE.ClipLimit = str2double(cell2mat(answer(2)));
handles.CLAHE.NBins = str2double(cell2mat(answer(3)));
handles.CLAHE.Distribution = cell2mat(answer(4));
handles.CLAHE.Alpha = str2double(cell2mat(answer(5)));
pause(.5);
if current_sw == 1    % do only for one frame
    ib_do_backup(handles, 'image', 0);
    sliceNo = handles.Img{handles.Id}.I.getCurrentSliceNumber();
    img = ib_getSlice('image', handles, sliceNo, NaN, colCh);
    start_no = 1;
    finish_no = start_no;
else                                    % do for all frames
    ib_do_backup(handles, 'image', 1);
    img = ib_getDataset('image', handles, NaN, colCh);
    start_no = 1;
    finish_no = size(img{1}, 4);
    sliceNo = handles.Img{handles.Id}.I.getCurrentSliceNumber();
end

Distribution = handles.CLAHE.Distribution;
NumTiles = handles.CLAHE.NumTiles;
ClipLimit = handles.CLAHE.ClipLimit;
NBins = handles.CLAHE.NBins;
Alpha = handles.CLAHE.Alpha;

if start_no==finish_no          % single slice
    for ind = 1:numel(img)
        if strcmp(Distribution,'uniform')
            img{ind} = adapthisteq(img{ind},...
                'NumTiles',NumTiles,'clipLimit',ClipLimit,'NBins', NBins,...
                'Distribution',Distribution);
        else
            img{ind} = adapthisteq(img{ind},...
                'NumTiles',NumTiles,'clipLimit',ClipLimit,'NBins', NBins,...
                'Distribution',Distribution,'Alpha', Alpha);
        end
    end
    ib_setSlice('image', img, handles, sliceNo, NaN, colCh);
else        % volume
    parfor ind = 1:numel(img)
        img2 = img{ind};
        for i=start_no:finish_no
            if strcmp(Distribution,'uniform')
                img2(:,:,1,i) = adapthisteq(img2(:,:,1,i),...
                    'NumTiles',NumTiles,'clipLimit',ClipLimit,'NBins', NBins,...
                    'Distribution',Distribution);
            else
                img2(:,:,1,i) = adapthisteq(img2(:,:,1,i),...
                    'NumTiles',NumTiles,'clipLimit',ClipLimit,'NBins', NBins,...
                    'Distribution',Distribution,'Alpha', Alpha);
            end
            % waitbar((i-start_id)/(finish_id-start_id),wb);
        end
        img{ind} = img2;
    end
    ib_setDataset('image', img, handles, NaN, colCh);
end

log_text = ['CLAHE; NumTiles: ' num2str(handles.CLAHE.NumTiles) ';clipLimit: ' num2str(handles.CLAHE.ClipLimit)...
    ';NBins:' num2str(handles.CLAHE.NBins) ';Distribution:' handles.CLAHE.Distribution ';Alpha:' num2str(handles.CLAHE.Alpha) ';ColCh:' num2str(colCh)];
handles.Img{handles.Id}.I.updateImgInfo(log_text);
delete(wb);
end
