function handles = ib_contrastNormalizationMemoryOptimized(handles, type_switch)
% function I = ib_contrastNormalizationMemoryOptimized(handles, type_switch)
% Normalize contrast between the layers of the dataset
%
% This function requires @em handles structure of im_browser to get slices/datasets directly. It may work in two modes: the
% whole dataset or slice-by-slice. After some tests the slice-by-slice mode seems to be faster and more memory friendly for
% most cases. So it is fixed now in that mode. To change that the useFullDatasetSwitch = 0; should be commented in the
% following section of the code.
% @code
% // use the slice-by-slice mode, because it seems to be faster for most cases
% // the whole dataset mode make sense for the datasets with a very many sections
% useFullDatasetSwitch = 0;
% @endcode
% Parameters:
% handles: -> handles of im_broser
% type_switch: a type of the normalization: 
%   - ''normal'' - normalize contrast using intensities of each complete slice
%   - ''mask'' - normalize contrast using intensities of only masked area at each slice
%   - ''bgMean'' - shift intensities of each image based on background
%   intensity that is marked as the mask
%
% Return values:
% handles: - handles of im_broser

%| For @em normal type:
% - calculate mean intensity and its std for the whole dataset
% - calculate mean intensities and std for each layer
% - shift each layer based on difference between mean values of each
% layer and the whole dataset, plus stretch based on ratio between std of
% the whole dataset and current layer
%
% For @em mask type:
% - calculate mean intensity for the masked or selected area for the whole dataset
% - calculate mean intensities for the masked or selected area for each layer
% - shift each layer based on difference between mean values of each
% layer and the whole dataset
%
% For @em bgMean type:
% - calculate mean intensity for the masked or selected area for the whole dataset
% - shift each slice by the mean intensity of the masked or selected areas

% Copyright (C) 04.12.2013, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 18.01.2016, added fix for memory function on linux and macos

if strcmp(handles.Img{handles.Id}.I.img_info('ColorType'),'indexed')
    msgbox(sprintf('Please convert to grayscale or truecolor data format first!\nMenu->Image->Mode->'),'Change format!','error','modal');
    return;
end

colorChannel = get(handles.ColChannelCombo,'Value') - 1;
if colorChannel == 0
    msgbox('Please select the color channel!','Error!','error','modal');
    return;
end

% define layer for normalization
if strcmp(type_switch, 'mask') || strcmp(type_switch, 'bgMean')
    button =  questdlg(sprintf('Define layer for normalization:'),'Selection or Mask','Selection','Mask','Cancel','Selection');
    switch button
        case 'Cancel'
            return;
        case 'Selection'
            layer_id = 'selection';
        case 'Mask'
            layer_id = 'mask';
    end
end

ib_do_backup(handles, 'image', 1);

% % detect whether it is possible to work with full dataset
% useFullDatasetSwitch = 0;
% if ispc   % memory is only available for windows
%     [userview, systemview] = memory;
%     datasetMemory = handles.Img{handles.Id}.I.getDatasetSizeInBytes();   % get physical size of datasets
%     if (systemview.PhysicalMemory.Available - userview.MemUsedMATLAB) / datasetMemory > 1
%         useFullDatasetSwitch = 1;
%     end
% end

%% --------------------
% use the slice-by-slice mode, because it seems to be faster for most cases
% the whole dataset mode make sense for the datasets with a very many sections
useFullDatasetSwitch = 0;

tic;
if useFullDatasetSwitch
    strTemp = sprintf('Normalizing whole dataset at once\nPlease wait...');
else
    strTemp = sprintf('Normalizing dataset slice by slice\nPlease wait...');    
end
wb = waitbar(0,strTemp,'Name','Normalizing layers...');

[maxH, maxW, maxC, maxZ] = handles.Img{handles.Id}.I.getDatasetDimensions('image', NaN);
counter = 1;

mean_val = zeros(maxZ, 1);
std_val = zeros(maxZ, 1);
waitbar(0.05, wb);

% check need for the mask
useMask = 0;     
if strcmp(type_switch, 'mask') || strcmp(type_switch, 'bgMean')
    useMask = 1;
    if strcmp(layer_id, 'mask')
        if handles.Img{handles.Id}.I.maskExist == 0
            msgbox(sprintf('Cancelled!\nNo mask information found!\n\nPlease draw Mask for each slice of the dataset and try again'),'Missing the Mask!','error','modal');
            delete(wb);
            return;
        end
    end
end

if useFullDatasetSwitch         % normalized by using full dataset, may be faster but less memory friendly
    I = handles.Img{handles.Id}.I.getData4D('image', NaN, NaN, colorChannel);
    mask = NaN;
    if useMask
        mask = handles.Img{handles.Id}.I.getData4D(layer_id, NaN, NaN);
    end
    
    % calculate sdt and mean
    for i=1:maxZ
        curr_img = I(:,:,1,i);
        
        if isnan(mask)
            mean_val(i) = mean2(curr_img);
            std_val(i) = std2(curr_img);
        else
            mean_val(i) = mean2(curr_img(mask(:,:,i)==1));
            std_val(i) = std2(curr_img(mask(:,:,i)==1));
        end

    end
    imgMean = mean(mean_val);
    imgStd = mean(std_val);
    
    if strcmp(type_switch, 'bgMean')    % intensities based on mean background value in the mask area
        for i=1:maxZ
            I(:,:,1,i) = double(I(:,:,1,i)) - mean_val(i) + imgMean;
            if mod(counter,10)==0; waitbar(counter/maxZ,wb); end;
            counter = counter + 1;
        end
    else
        for i=1:maxZ
            ratio = imgStd/std_val(i);
            I1 = double(I(:,:,1,i)) - mean_val(i);
            I1 = I1 * ratio;
            I(:,:,1,i) = I1 + imgMean;
            if mod(counter,10)==0; waitbar(counter/maxZ,wb); end;
            counter = counter + 1;
        end
    end
    handles.Img{handles.Id}.I.setData4D('image', I, NaN, NaN, colorChannel);
else        % % normalized slice by slice, may be slower but memory friendly
    % calculate sdt and mean
    for i=1:maxZ
        curr_img = handles.Img{handles.Id}.I.getSlice('image', i, NaN, colorChannel);
        
        if useMask == 0
            mean_val(i) = mean2(curr_img);
            std_val(i) = std2(curr_img);
        else
            mask = handles.Img{handles.Id}.I.getSlice(layer_id, i);
            mean_val(i) = mean2(curr_img(mask==1));
            std_val(i) = std2(curr_img(mask==1));
        end
    end
    imgMean = mean(mean_val);
    imgStd = mean(std_val);
    
    if strcmp(type_switch, 'bgMean')    % intensities based on mean background value in the mask area
        for i=1:maxZ
            curr_img = handles.Img{handles.Id}.I.getSlice('image', i, NaN, colorChannel);
            curr_img = double(curr_img) - mean_val(i) + imgMean;
            handles.Img{handles.Id}.I.setSlice('image', curr_img, i, NaN, colorChannel);
            if mod(counter,10)==0; waitbar(counter/maxZ,wb); end;
            counter = counter + 1;
        end
    else
        for i=1:maxZ
            ratio = imgStd/std_val(i);
            curr_img = handles.Img{handles.Id}.I.getSlice('image', i, NaN, colorChannel);
            I1 = double(curr_img) - mean_val(i);
            I1 = I1 * ratio;
            curr_img = I1 + imgMean;
            handles.Img{handles.Id}.I.setSlice('image', curr_img, i, NaN, colorChannel);
            if mod(counter,10)==0; waitbar(counter/maxZ,wb); end;
            counter = counter + 1;
        end
    end
end
% update the log
log_text = sprintf('Normalize contrast, mode: %s, Ch: %d', type_switch, colorChannel);
handles.Img{handles.Id}.I.updateImgInfo(log_text);

delete(wb);
toc
end