function [img, logText] = ib_doImageFiltering(img, options)
% function [img, logText] = ib_doImageFiltering(img, options)
% Filter image with basic Matlab filters
%
% Parameters:
% img: -> image stack, [1:height, 1:width, 1:color, 1:layers]
% options: -> a structure with parameters
% - .dataType - type of the dataset, ''4D'' - [height, width, color, z], ''3D'' - [height, width, z]
% - .fitType - type of the filter to use: ''Gaussian 3d'', ''Gaussian'', ''Disk'', ''Motion'', ''Unsharp'',''Median
% 2D'',''Wiener 2D'', ''Average'', ''Motion''
% - .colorChannel - color channel to filter, when @b 0 filter all channels
% - .hSize - size of the Kernel to use
% - .sigma - sigma for the filters (if required)
% - .lambda - lambda for the filters (if required)
% - .pixSize - voxel sizes for 3D gaussian (imageData.pixSize.x imageData.pixSize.y imageData.pixSize.z)
% - .orientation - orientation of the dataset for 2D filters,  1-for XZ, 2 - YZ, 4  - XY
%
% Return values:
% img: filtered dataset, [1:height, 1:width, 1:color, 1:no_stacks]
% logText: log text with parameters of the applied filter

% Copyright (C) 21.11.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

logText = NaN;

if isfield(options,'sigma') && options.sigma <= 0; error('ib_doImageFiltering: Sigma should be above zero!'); end;
if ~isfield(options, 'pixSize') && strcmp(options.fitType, 'Gaussian 3d')
    error('ib_doImageFiltering: such call requires specification of pixSize!');
end
if ~isfield(options, 'colorChannel'); options.colorChannel = 0; end

wb = waitbar(0,['Applying ' options.fitType ' filter...'],'Name','Filtering','WindowStyle','modal');

if strcmp(options.dataType, '3D')   % reshape image, for selection and mask
    img = reshape(img,[size(img,1), size(img,2), 1, size(img,3)]);
end

if options.colorChannel == 0
    color_start = 1;
    color_end = size(img,3);
else
    if size(img,3) == 1
        color_start = 1;
        color_end = 1;
    else
        color_start = options.colorChannel;
        color_end = options.colorChannel;
    end
end
matlabVersion = ver('Matlab');
matlabVersion = str2double(matlabVersion.Version);

if strcmp(options.fitType,'Gaussian 3D')       % 3d gaussian filter
    if size(img,4) < 3
        delete(wb);
        msgbox('Gaussian 3D filter requires more than 2 layers!','Error!','error');
        return;
    end
    if numel(options.hSize) == 1
        z_kernel =  round(options.hSize(1)*options.pixSize.x/options.pixSize.z);     % kernel dimensions for z-direction
    else
        z_kernel = options.hSize(2);
    end
    for color_ch=color_start:color_end
        GaussXY =  fspecial('gaussian',[options.hSize(1) 1],options.sigma);
        GaussZ =  fspecial('gaussian',[z_kernel 1],options.sigma);
        Hx=reshape(GaussXY,[length(GaussXY) 1 1 1]);
        Hy=reshape(GaussXY,[1 length(GaussXY) 1 1]);
        Hz=reshape(GaussZ,[1 1 1 length(GaussZ)]);
        img(:,:,color_ch,:) = imfilter(imfilter(imfilter(img(:,:,color_ch,:),Hx,'replicate'),Hy,'replicate'),Hz,'replicate');
    end
    if strcmp(options.dataType, '3D')   % reshape image, for selection and mask
        img = squeeze(img);
    end
    logText = ['ImFilter: ' options.fitType ', HSize:' num2str(options.hSize) ...
        ', ZSize:' num2str(z_kernel) ', Sigma: ' num2str(options.sigma) ',ColCh:' num2str(options.colorChannel)];
elseif strcmp(options.fitType,'Gradient 3D')       % 3d gradient filter
    if size(img,4) < 3
        delete(wb);
        msgbox('Gradient 3D filter requires more than 2 layers!','Error!','error');
        return;
    end
    for color_ch=color_start:color_end
        [Ix,Iy,Iz] = gradient(double(squeeze(img(:,:,color_ch,:))));
        img2 = sqrt(Ix.^2 + Iy.^2 + Iz.^2);
        img(:,:,color_ch,:) = img2/max(max(max(img2)))*double(intmax(class(img)));      % convert to data type
    end
    if strcmp(options.dataType, '3D')   % reshape image, for selection and mask
        img = squeeze(img);
    end
    logText = ['ImFilter: ' options.fitType ', ColCh:' num2str(options.colorChannel)];
else    % 2D filters
    if strcmp(options.fitType,'Disk') || strcmp(options.fitType,'Motion') || strcmp(options.fitType,'Unsharp')
        options.hSize = options.hSize(1);
    end
    
    max_layers = size(img, options.orientation);
    
    % generate filter and log_text
    switch options.fitType
        case {'Median 2D', 'Wiener 2D'}
            log_text = ['ImFilter: ' options.fitType ', HSize:' num2str(options.hSize) ...
                ',Orient:' num2str(options.orientation) ',ColCh:' num2str(options.colorChannel)];
        case 'Unsharp'
            if matlabVersion >= 8.1
                log_text = ['ImFilter: ' options.fitType ', Radius:' num2str(options.hSize) ...
                    ',Amount:' num2str(options.sigma) ',Thres:' num2str(options.lambda) ...
                    ',Orient:' num2str(options.orientation) ',ColCh:' num2str(options.colorChannel)];
            else
                filter2d = fspecial(options.fitType, options.hSize);
                log_text = ['ImFilter: ' options.fitType ', HSize:' num2str(options.hSize) ...
                    ',Orient:' num2str(options.orientation) ',ColCh:' num2str(options.colorChannel)];
            end
        case 'Gradient 2D'
            log_text = ['ImFilter: ' options.fitType ',ColCh:' num2str(options.colorChannel)];
        otherwise
            if isfield(options, 'sigma') && isfield(options, 'hSize')
                filter2d = fspecial(options.fitType, options.hSize, options.sigma);
                log_text = ['ImFilter: ' options.fitType ', HSize:' num2str(options.hSize) ...
                    ', Sigma: ' num2str(options.sigma) ',Orient:' num2str(options.orientation) ',ColCh:' num2str(options.colorChannel)];
            elseif isfield(options, 'sigma')
                filter2d = fspecial(options.fitType, options.sigma);
                log_text = ['ImFilter: ' options.fitType, ',Sigma: ' num2str(options.sigma) ',Orient:' num2str(options.orientation) ',ColCh:' num2str(options.colorChannel)];
            else
                filter2d = fspecial(options.fitType, options.hSize);
                log_text = ['ImFilter: ' options.fitType ', HSize:' num2str(options.hSize) ...
                    ',Orient:' num2str(options.orientation) ',ColCh:' num2str(options.colorChannel)];
            end
    end
    
    if ndims(img) > 3   % full dataset
        if options.orientation == 4     % xy plane
            for id = 1:size(img, options.orientation)
                % each pixel in the result is a sum of neightbour pixels multiplied
                % with coefficients provided in filter2d
                for color_ch=color_start:color_end
                    switch options.fitType
                        case 'Median 2D'
                            img(:,:,color_ch,id) = medfilt2(img(:,:,color_ch,id), options.hSize, 'symmetric');
                        case 'Wiener 2D'
                            img(:,:,color_ch,id) = wiener2(img(:,:,color_ch,id),options.hSize);
                        case 'Gradient 2D'
                            [Ix,Iy] = gradient(double(img(:,:,color_ch,id)));
                            img2 = sqrt(Ix.^2 + Iy.^2);
                            img(:,:,color_ch,id) = img2/max(max(max(img2)))*double(intmax(class(img)));  
                        case 'Unsharp'
                            if matlabVersion >= 8.1
                                img(:,:,color_ch,id) = imsharpen(img(:,:,color_ch,id),'Radius',options.hSize,'Amount',options.sigma, 'Threshold', options.lambda);
                            else
                                img(:,:,color_ch,id) = imfilter(img(:,:,color_ch,id), filter2d, 'replicate');
                            end
                        case 'Log'
                            tempImg = imfilter(double(img(:,:,color_ch,id)), filter2d, 'replicate');
                            minVal = min(min(tempImg));
                            maxVal = max(max(tempImg));
                            img(:,:,color_ch,id) = (tempImg-minVal)/(maxVal-minVal)*double(intmax(class(img)));
                        otherwise
                            img(:,:,color_ch,id) = imfilter(img(:,:,color_ch,id), filter2d, 'replicate');
                    end
                end
                if mod(id, 10)==0; waitbar(id/max_layers,wb); end;
            end
        elseif options.orientation == 1     % xz plane
            for id = 1:size(img, options.orientation)
                for color_ch=color_start:color_end
                    switch options.fitType
                        case 'Median 2D'
                            img(id,:,color_ch,:) = medfilt2(squeeze(img(id,:,color_ch,:)), options.hSize, 'symmetric');
                        case 'Wiener 2D'
                            img(id,:,color_ch,:) = wiener2(squeeze(img(id,:,color_ch,:)),options.hSize);
                        case 'Gradient 2D'
                            [Ix,Iy] = gradient(double(squeeze(img(id,:,color_ch,:))));
                            img2 = sqrt(Ix.^2 + Iy.^2);
                            img(id,:,color_ch,:) = img2/max(max(max(img2)))*double(intmax(class(img)));                              
                        case 'Unsharp'
                            if matlabVersion >= 8.1
                                img(id,:,color_ch,:) = imsharpen(squeeze(img(id,:,color_ch,:)),'Radius',options.hSize,'Amount',options.sigma, 'Threshold', options.lambda);
                            else
                                img(id,:,color_ch,:) =  imfilter(squeeze(img(id,:,color_ch,:)), filter2d, 'replicate');
                            end
                        case 'Log'
                            tempImg = imfilter(double(squeeze(img(id,:,color_ch,:))), filter2d, 'replicate');
                            minVal = min(min(tempImg));
                            maxVal = max(max(tempImg));
                            img(id,:,color_ch,:) = (tempImg-minVal)/(maxVal-minVal)*double(intmax(class(img)));
                        otherwise
                            img(id,:,color_ch,:) = imfilter(squeeze(img(id,:,color_ch,:)), filter2d, 'replicate');
                    end
                    if mod(id, 10)==0; waitbar(id/max_layers,wb); end;
                end
            end
        elseif options.orientation == 2     % yz plane
            for id = 1:size(img, options.orientation)
                for color_ch=color_start:color_end
                    switch options.fitType
                        case 'Median 2D'
                            img(:,id,color_ch,:) = medfilt2(squeeze(img(:,id,color_ch,:)), options.hSize, 'symmetric');
                        case 'Wiener 2D'
                            img(:,id,color_ch,:) = wiener2(squeeze(img(:,id,color_ch,:)),options.hSize);
                        case 'Gradient 2D'
                            [Ix,Iy] = gradient(double(squeeze(img(:,id,color_ch,:))));
                            img2 = sqrt(Ix.^2 + Iy.^2);
                            img(:,id,color_ch,:) = img2/max(max(max(img2)))*double(intmax(class(img)));    
                        case 'Unsharp'
                            if matlabVersion >= 8.1
                                img(:,id,color_ch,:) = imsharpen(squeeze(img(:,id,color_ch,:)),'Radius',options.hSize,'Amount',options.sigma, 'Threshold', options.lambda);
                            else
                                img(:,id,color_ch,:) = imfilter(squeeze(img(:,id,color_ch,:)), filter2d, 'replicate');
                            end
                        case 'Log'
                            tempImg = imfilter(double(squeeze(img(:,id,color_ch,:))), filter2d, 'replicate');
                            minVal = min(min(tempImg));
                            maxVal = max(max(tempImg));
                            img(:,id,color_ch,:) = (tempImg-minVal)/(maxVal-minVal)*double(intmax(class(img)));
                        otherwise
                            img(:,id,color_ch,:) = imfilter(squeeze(img(:,id,color_ch,:)), filter2d, 'replicate');
                    end
                    
                end
                if mod(id, 10)==0; waitbar(id/max_layers,wb); end;
            end
        else
            error('Wrong orientation');
        end
        logText = log_text;
    else    % a single slice
        for color_ch=color_start:color_end
            switch options.fitType
                case 'Median 2D'
                    img(:,:,color_ch) = medfilt2(img(:,:,color_ch), options.hSize, 'symmetric');
                case 'Wiener 2D'
                    img(:,:,color_ch) = wiener2(img(:,:,color_ch),options.hSize);
                case 'Gradient 2D'
                    [Ix,Iy] = gradient(double(img(:,:,color_ch)));
                    img2 = sqrt(Ix.^2 + Iy.^2);
                    img(:,:,color_ch) = img2/max(max(max(img2)))*double(intmax(class(img)));
                case 'Unsharp'
                    if matlabVersion >= 8.1
                        img(:,:,color_ch) = imsharpen(img(:,:,color_ch),'Radius',options.hSize,'Amount',options.sigma, 'Threshold', options.lambda);
                    else
                        img(:,:,color_ch) =  imfilter(img(:,:,color_ch), filter2d, 'replicate');
                    end
                case 'Log'
                    tempImg = imfilter(double(img(:,:,color_ch)), filter2d, 'replicate');
                    minVal = min(min(tempImg));
                    maxVal = max(max(tempImg));
                    img(:,:,color_ch) = (tempImg-minVal)/(maxVal-minVal)*double(intmax(class(img)));
                otherwise
                    img(:,:,color_ch) = imfilter(img(:,:,color_ch), filter2d, 'replicate');
            end
        end
        logText = log_text;
    end
    
    if strcmp(options.dataType, '3D')  % reshape image, for selection and mask
        img = squeeze(img);
    end
    
    
end
delete(wb);
end