function handles = ib_setImarisDataset(img, img_info, handles)
% function [img, img_info, viewPort] = ib_getImarisDataset(handles)
% Send a dataset from MIB to Imaris
%
% Parameters:
% img:  a variable with the dataset
% img_info: containers.map with parameters
% handles: handles structure from im_browser
%
% Return values:
% handles:  handles structure from im_browser

% @note
% uses IceImarisConnector bindings
% @b Requires:
% 1. set system environment variable IMARISPATH to the installation
% directory, for example "c:\tools\science\imaris"
% 2. restart Matlab

%|
% @b Examples:
% @code handles = ib_setImarisDataset(handles.Img{handles.Id}.I.img, handles.Img{handles.Id}.I.img_info, handles);     // get dataset from imaris @endcode

% Copyright (C) 05.11.2014 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

% establish connection to Imaris
wb = waitbar(0, 'Please wait...', 'Name', 'Connecting to Imaris');
if ~isfield(handles, 'connImaris')
    try
        handles.connImaris = IceImarisConnector(0);
        waitbar(0.3, wb);
    catch exception
        %if strcmp(exception.message, 'Could not connect to Imaris Server.')
        errordlg(sprintf('Could not connect to Imaris Server;\nPlease start Imaris and try again!'),'Missing Imaris');
        delete(wb);
        return;
    end
    waitbar(0.7, wb);
    if handles.connImaris.isAlive == 0
        % start Imaris
        handles.connImaris.startImaris()
        waitbar(0.95, wb);
    end
else
    if handles.connImaris.isAlive == 0
        handles.connImaris.startImaris();
    end
end
delete(wb);

sizeX = size(img,2);
sizeY = size(img,1);
sizeZ = size(img,4);

blockSizeX = 512;
blockSizeY = 512;
blockSizeZ = 512;

useBlockMode = 0;
if sizeX*sizeY*sizeZ > 134217728 % = 512 x 512 x 512
    useBlockMode = 1;
end

% check whether replace the dataset or update a slice
if isempty(handles.connImaris.mImarisApplication.GetDataSet)
    timePoint = -1;     % replace dataset
else
    [vSizeX, vSizeY, vSizeZ, vSizeC, vSizeT] = handles.connImaris.getSizes();
    if vSizeZ > 1 && vSizeT > 1
        %answer = inputdlg(sprintf('!!! Warning !!!\n\nA 5D dataset is open in Imaris!\nPlease enter a time point to update (starting from 0)\nor type "-1" to replace dataset completely'), 'Time point', 1, cellstr('0'));
        answer = mib_inputdlg(NaN,sprintf('!!! Warning !!!\n\nA 5D dataset is open in Imaris!\nPlease enter a time point to update (starting from 0)\nor type "-1" to replace dataset completely'), 'Time point', '0');
        if isempty(answer)
            return;
        end
        timePoint = str2double(answer{1});    % frame number to open for 5D datasets
    else
        timePoint = -1;
    end
end

wb = waitbar(0, 'Please wait...','Name','Export image to Imaris');
callsId = 0;
maxWaitbarIndex = size(img,3);
if timePoint == -1
    % create an empty dataset
    handles.connImaris.createDataset(class(img), size(img,2), size(img,1), size(img,4), size(img,3), 1);
    % update colorchannels
    colorData = get(handles.channelMixerTable,'data');   % get data in the channelMixerTable
    for colId = 1:size(img,3)
        % set dataset as a new
        if useBlockMode == 0
            handles.connImaris.setDataVolumeRM(squeeze(img(:,:,colId,:)), colId-1, 0);
            callsId = callsId + 1;
        else
            maxWaitbarIndex = size(img,3)*ceil(sizeZ/blockSizeZ)*ceil(sizeY/blockSizeY)*ceil(sizeX/blockSizeX);
            for z=0:ceil(sizeZ/blockSizeZ)-1
                for y=0:ceil(sizeY/blockSizeY)-1
                    for x=0:ceil(sizeX/blockSizeX)-1
                        imgBlock = squeeze(img(...
                            1+blockSizeY*y:min(blockSizeY+blockSizeY*y, sizeY) ,...
                            1+blockSizeX*x:min(blockSizeX+blockSizeX*x, sizeX) ,...
                            colId,...
                            1+blockSizeZ*z:min(blockSizeZ+blockSizeZ*z, sizeZ)));
                        
                        handles.connImaris.mib_setDataSubVolumeRM(imgBlock,...
                            blockSizeX*x, blockSizeY*y, blockSizeZ*z,...
                            colId-1, 0,...
                            size(imgBlock,2), size(imgBlock,1), size(imgBlock,3));
                        callsId = callsId + 1;
                        waitbar(callsId/maxWaitbarIndex, wb);
                    end
                end
            end
        end
                
        handles.connImaris.mImarisApplication.GetDataSet.SetChannelRange(colId-1, ...
            handles.Img{handles.Id}.I.viewPort.min(colId), handles.Img{handles.Id}.I.viewPort.max(colId));
        handles.connImaris.mImarisApplication.GetDataSet.SetChannelGamma(colId-1, handles.Img{handles.Id}.I.viewPort.gamma(colId));
        
        ColorRGBA = colorData{colId,3};
        index1 = strfind(ColorRGBA, 'rgb(');    % example: '<html><table border=0 width=40 bgcolor='rgb(0, 0, 0)'><TR><TD>&nbsp;</TD></TR> </table></html>'
        index2 = strfind(ColorRGBA, '><TR>');
        ColorRGBA = ColorRGBA(index1+4:index2-3);
        %ColorRGBA = str2num(ColorRGBA)/double(intmax(class(img))); %#ok<ST2NM>
        ColorRGBA = str2num(ColorRGBA)/255; %#ok<ST2NM>
        
        % replace black with white
        for i=1:size(ColorRGBA,1)
            if sum(ColorRGBA(i,:)) == 0
                ColorRGBA(i,:) = [1 1 1];
            end
        end
        
        ColorRGBA(4) = 0;   % add Alpha value
        ColorRGBA = handles.connImaris.mapRgbaVectorToScalar(ColorRGBA);
        handles.connImaris.mImarisApplication.GetDataSet.SetChannelColorRGBA(colId-1, ColorRGBA);     % update color channel
        waitbar(callsId/maxWaitbarIndex, wb);
    end
    updateBoundingBox = 1;  % switch to update bouning box
else
    for colId = 1:vSizeC
        if useBlockMode == 0
            handles.connImaris.setDataVolumeRM(squeeze(img(:,:,colId,:)), colId-1, timePoint);
            callsId = callsId + 1;
        else
            maxWaitbarIndex = size(img,3)*ceil(sizeZ/blockSizeZ)*ceil(sizeY/blockSizeY)*ceil(sizeX/blockSizeX);
            for z=0:ceil(sizeZ/blockSizeZ)-1
                for y=0:ceil(sizeY/blockSizeY)-1
                    for x=0:ceil(sizeX/blockSizeX)-1
                        imgBlock = squeeze(img(...
                            1+blockSizeY*y:min(blockSizeY+blockSizeY*y, sizeY) ,...
                            1+blockSizeX*x:min(blockSizeX+blockSizeX*x, sizeX) ,...
                            colId,...
                            1+blockSizeZ*z:min(blockSizeZ+blockSizeZ*z, sizeZ)));
                        
                        handles.connImaris.mib_setDataSubVolumeRM(imgBlock,...
                            blockSizeX*x, blockSizeY*y, blockSizeZ*z,...
                            colId-1, timePoint,...
                            size(imgBlock,2), size(imgBlock,1), size(imgBlock,3));
                        callsId = callsId + 1;
                        waitbar(callsId/maxWaitbarIndex, wb);
                    end
                end
            end
        end
    end
end

% update BoundingBox and Image Description
if updateBoundingBox == 1
    bb = handles.Img{handles.Id}.I.getBoundingBox();    % bb[xMin, xMax, yMin, yMax, zMin, zMax]
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMinX(bb(1));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxX(bb(2));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMinY(bb(3));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxY(bb(4));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMinZ(bb(5));
    % fix of a problem of different calculations of the bounding box for a
    % single slice and Z-stack
    if size(img,4) > 1
        handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxZ(bb(6)+handles.Img{handles.Id}.I.pixSize.z);
    else
        handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxZ(bb(6));
    end
    
    logText = handles.Img{handles.Id}.I.img_info('ImageDescription');
    linefeeds = strfind(logText,sprintf('|'));
    
    if ~isempty(linefeeds)
        for linefeed = 1:numel(linefeeds)
            if linefeed == 1
                logTextForm(linefeed) = cellstr(logText(1:linefeeds(1)-1)); %#ok<AGROW>
            else
                logTextForm(linefeed) = cellstr(logText(linefeeds(linefeed-1)+1:linefeeds(linefeed)-1)); %#ok<AGROW>
            end
        end
        if numel(logText(linefeeds(end)+1:end)) > 1
            logTextForm(linefeed+1) = cellstr(logText(linefeeds(end)+1:end));
        end
    else
		logTextForm = [];
	end
    logOut = [];
    for i=1:numel(logTextForm)
        logOut = [logOut sprintf('%s\n',logTextForm{i})];
    end
    handles.connImaris.mImarisApplication.GetDataSet.SetParameter('Image','Description',logOut);
end
% set the time point for the dataset to sync it later with models
handles.connImaris.mImarisApplication.GetDataSet.SetTimePoint(0, '2006-05-11 13:30:00.000');
delete(wb);
end
