function result = image2hdf5(filename, imageS, options, ImageDescription)
% function result = image2hdf5(filename, imageS, options, ImageDescription)
% Save image into hdf5 format
%
% Parameters:
% filename: filename for hdf file
% imageS: original dataset [1:height, 1:width, 1:colors, 1:no_stacks] or [1:height, 1:width, 1:no_stacks]
% options: [@em optional] a structure with additional parameters
%  - .overwrite, if @b 1 do not check whether file with provided filename alaready exists
%  - .showWaitbar: @b 1 - show the progress bar, @b 0 - do not show
% ImageDescription: - a cell string or a containers.Map with metadata
%
% Return values:
% result: result of the function run, @b 1 - success, @b 0 - fail

% Copyright (C) 19.01.2012 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

% example:
%   image2hdf5('saveme.h5', image_var, options, 'test');

result = 0;
if nargin < 2
    error('Please provide filename and image!');
end

if nargin < 4
    ImageDescription = containers.Map;
end

if ischar(ImageDescription)
    stringIn = ImageDescription;
    ImageDescription = containers.Map;
    ImageDescription('ImageDescription') = cellstr(stringIn);
end

if nargin < 3
    options.showWaitbar = 1;
    options.overwrite = 0;
end

if ~isfield(options, 'showWaitbar');     options.showWaitbar = 1;   end;
if ~isfield(options, 'overwrite');     options.overwrite = 0;  end;

if options.overwrite == 0
    if exist(filename,'file') == 2
        reply = input('File exists! Overwrite? [y/N]:','s');
        if ~strcmp(reply,'y'); disp('Cancel, nothing was saved!'); return; end;
    end;
end
if numel(size(imageS)) == 3 && size(imageS,3) ~= 3
    imageS = reshape(imageS,size(imageS,1), size(imageS,2), 1, size(imageS,3));
end

if options.showWaitbar; 
    %warning('off','MATLAB:gui:latexsup:UnableToInterpretTeXString');    % switch off warnings for latex
	curInt = get(0, 'DefaulttextInterpreter'); 
	set(0, 'DefaulttextInterpreter', 'none'); 
    wb = waitbar(0,sprintf('%s\nPlease wait...',filename),'Name','Saving images as hdf5...','WindowStyle','modal');
    waitbar(0, wb); 
end;

% Set dataset name and check for the leading slash
if ~isKey(ImageDescription,'DatasetName')
    ImageDescription('DatasetName') = '/MatlabData';
else
    DatasetName = ImageDescription('DatasetName');
    if DatasetName(1) ~= '/'
        ImageDescription('DatasetName') = ['/' DatasetName];
    end
end

if isKey(ImageDescription,'SliceName'); remove(ImageDescription,'SliceName'); end;  % remove slice name, because it is a cell array

if exist(filename,'file')   % for overwrite
    fileNameDelete = filename;
    delete(fileNameDelete);
end
if options.showWaitbar; waitbar(.05,wb,sprintf('%s\nCreate file container...',filename)); end;
h5create(filename,ImageDescription('DatasetName'), [size(imageS,1) size(imageS,2) size(imageS,3), size(imageS,4)], 'Datatype', class(imageS),...
    'Deflate', 9, 'ChunkSize',[ceil(size(imageS,1)/25) ceil(size(imageS,2)/25) ceil(size(imageS,3)/25) ceil(size(imageS,4)/25)]);
if options.showWaitbar; waitbar(.1,wb,sprintf('%s\nSaving images...',filename)); end;
h5write(filename, ImageDescription('DatasetName'), imageS);
if options.showWaitbar; waitbar(.9,wb,sprintf('%s\nSaving metadata...',filename)); end;
metaFields = keys(ImageDescription);
for index=1:numel(metaFields)
    h5writeatt(filename,ImageDescription('DatasetName'), metaFields{index}, ImageDescription(metaFields{index}));
end

waitbar(1);
disp(['image2hdf5: ' filename ' was created!']);
if options.showWaitbar; delete(wb); set(0, 'DefaulttextInterpreter', curInt); end;
result = 1;
end