function handles = otsuThresholding(par1, par2, h_im_browser)
% par1, par2 will not be used
% h_im_browser - is a handle to the main program (im_browser)

% Copyright (C) 21.11.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

% get the handles structure of the main program
handles = guidata(h_im_browser);

% get number of shown color channels
col_channels = handles.Img{handles.Id}.I.slicesColor;
if numel(col_channels) ~= 1;
    msgbox(sprintf('Warning!\nYou need to select a color channel for the thresholding.\nPlease keep only one channel in the View Settings Panel->Colors...'),'Wrong number of color channels!','Error');
    return;
end

% ask whether to threshold a single slice or a whole dataset
button = questdlg('Would you like to threshold a whole dataset or a single slice?','Otsu thresholding','Whole dataset','Single slice','Cancel','Whole dataset');
if strcmp(button, 'Cancel'); return; end;

% preallocate space for the mask if it is not present
if handles.Img{handles.Id}.I.maskExist == 0
    handles.Img{handles.Id}.I.clearMask();
else
    % store the existing Mask, so that the thresholding may be undone with Ctrl+Z
    handles = ib_do_backup(handles, 'mask', 1);
end


if strcmp(button, 'Single slice')
    img = ib_getSlice('image', handles);   
    % the 'image' parameter defines that the image is required, alternatively 
    % may be: 'model', 'mask', or 'selection'
    % when other parameters omitted the function call will take the 
    % currently shown slice. The function returns a cell with image or 
    % an array with cells when several ROIs present.
    
    % do the Otsu thresholding, 
    % the for-loop is needed because the image is return as a cell-array.
    % When ROIs are shown number of elements in the array is equial to
    % number of ROIs. When ROIs are not present the cell-array has only one
    % element.
    for roi=1:numel(img)
        level = graythresh(img{roi});  
        % use the same variable to store the thresholded image
        img{roi} = im2bw(img{roi},level);
    end
    
    % update the Mask layer with a new mask
    ib_setSlice('mask', img, handles);
end

if strcmp(button, 'Whole dataset')
    % Method A, get copy of the complete dataset
    img = ib_getDataset('image', handles); 
    % do the Otsu thresholding 
    for roi=1:numel(img)
        for slice = 1:size(img{1},4)
            level = graythresh(img{roi}(:,:,:,slice));  
            % use the same variable to store the thresholded image
            img{roi}(:,:,:,slice) = im2bw(img{roi}(:,:,:,slice),level);
        end
    end

    % Method B, threshold not the whole dataset but slice by slice. It is slower but requres less memory
    orientation = 4;    % define orientation to use: 1, 2 or 4
    for slice = 1:handles.Img{handles.Id}.I.no_stacks
        img = ib_getSlice('image', handles, slice, orientation);   
        for roi=1:numel(img)
            level = graythresh(img{roi});  
            img{roi} = im2bw(img{roi},level);
        end
        ib_setSlice('mask', img, handles, slice, orientation);
    end
end

% set the Show Mask check box. Or it may be pressed manually
set(handles.maskShowCheck,'value',1);

% update widgets in the im_browser GUI
handles = updateGuiWidgets(handles);

% redraw the im_browser 
handles = handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);


