function handles = ib_flipDataset(handles, mode)
% function handles = ib_flipDataset(handles, mode)
% Flip dataset horizontally, vertically or in the Z direction
% 
% Flip dataset and other layers in horizontal or vertical direction  
%
% Parameters:
% handles: handles structire of im_browser
% mode: -> a string that defines the flipping mode
%     - ''flipH'' -> horizontal flip
%     - ''flipV'' -> vertical flip
%     - ''flipZ'' -> flip Z direction
%
% Return values:
% handles: handles structure of im_browser

% Copyright (C) 25.06.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 
% 07.09.2015, IB, updated to use imageData.getData4D methods

tic
options.blockModeSwitch = 0;    % overwrite blockmode switch

wb = waitbar(0,sprintf('Flipping image\nPlease wait...'),'Name','Flip dataset','WindowStyle','modal');

ib_do_backup(handles, 'image', 1);

% flip image
img = handles.Img{handles.Id}.I.getData4D('image', NaN, 4, 0, options);   % get dataset (image)
img = flipme(img, mode);
handles.Img{handles.Id}.I.setData4D('image', img, NaN, 4, 0, options);   % set dataset (image) back
clear img;

% flip other layers
if strcmp(handles.Img{handles.Id}.I.model_type, 'uint6')
    waitbar(0.5, wb, sprintf('Flipping other layers\nPlease wait...'));
    ib_do_backup(handles, 'everything', 1);  % backup other layers
    img = handles.Img{handles.Id}.I.getData4D('everything', NaN, 4, NaN, options);   % get dataset (image)
    img = flipme(img, mode);
    handles.Img{handles.Id}.I.setData4D('everything', img, NaN, 4, NaN, options);   % set dataset (image) back
else
    % flip selection layer
    waitbar(0.25, wb, sprintf('Flipping the selection layer\nPlease wait...'));
    img = handles.Img{handles.Id}.I.getData4D('selection', NaN, 4, NaN, options);   % get dataset (image)
    img = flipme(img, mode);
    handles.Img{handles.Id}.I.setData4D('selection', img, NaN, 4, NaN, options);   % set dataset (image) back
    
    % flip mask
    if handles.Img{handles.Id}.I.maskExist
        waitbar(0.5, wb, sprintf('Flipping the mask layer\nPlease wait...'));
        img = handles.Img{handles.Id}.I.getData4D('mask', NaN, 4, NaN, options);   % get dataset (image)
        img = flipme(img, mode);
        handles.Img{handles.Id}.I.setData4D('mask', img, NaN, 4, NaN, options);   % set dataset (image) back
    end
    
    % flip model
    if handles.Img{handles.Id}.I.modelExist
        waitbar(0.75, wb, sprintf('Flipping the model layer\nPlease wait...'));
        img = handles.Img{handles.Id}.I.getData4D('model', NaN, 4, NaN, options);   % get dataset (image)
        img = flipme(img, mode);
        handles.Img{handles.Id}.I.setData4D('model', img, NaN, 4, NaN, options);   % set dataset (image) back
    end
end
waitbar(1, wb, sprintf('Finishing...'));

log_text = ['Flip: mode=' mode];
handles.Img{handles.Id}.I.updateImgInfo(log_text);

delete(wb);
toc
end

function img = flipme(img, mode)
% flip function
if strcmp(mode, 'flipZ')
    img = flipdim(img, ndims(img));
elseif strcmp(mode, 'flipH')
    img = flipdim(img, 2);
elseif strcmp(mode, 'flipV')
    img = flipdim(img, 1);
end
% else  % not compatible with images with more that 3 color channels
%     if ndims(img) == 4  % for image
%         for slice = 1:size(img, ndims(img))
%             if strcmp(mode, 'flipH')
%                 img(:,:,:,slice) = fliplr(img(:,:,:,slice));
%             elseif strcmp(mode, 'flipV')
%                 img(:,:,:,slice) = flipud(img(:,:,:,slice));
%             end
%         end
%     else    % for other layers
%         for slice = 1:size(img, ndims(img))
%             if strcmp(mode, 'flipH')
%                 img(:,:,slice) = fliplr(img(:,:,slice));
%             elseif strcmp(mode, 'flipV')
%                 img(:,:,slice) = flipud(img(:,:,slice));
%             end
%         end
%     end
% end
end