function handles = ib_sizeExclusionFilter(handles, type)
% function handles = ib_sizeExclusionFilter(handles, type)
% Apply size exclusion filter on selection, model, or mask layers
%
% Parameters:
% handles: structure with handles of im_browser.m
% type: a type of the layer for the size exclusion filter:
% - ''selection'' - run size exclusion on the 'Selection' layer
% - ''model'' - - run size exclusion on the 'Model' layer
% - ''mask'' - - run size exclusion on the 'Mask' layer
%
% Return values:
% handles: structure with handles of im_browser.m

% Copyright (C) 21.11.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


title = 'Size filtration of objects';
switch type
    case 'selection'
        prompt = {sprintf('Doing for selection (red areas)\n\nKeep objects larger than, px:'),...
                    'Keep objects smaller than, px:','Current layer only [1-current, 0-all]:',sprintf('3d (Put 0 for 2d mode;\n6, 18, or 26 connection options for 3d)')};
    case 'model'
        sel_model = get(handles.segmSelList,'Value')-2;
        prompt = {sprintf('Doing for model No: %d\n\nKeep objects larger than, px:',sel_model),...
                    'Keep objects smaller than, px:','Current layer only [1-current, 0-all]:',sprintf('3d (Put 0 for 2d mode;\n6, 18, or 26 connection options for 3d)')};
    case 'mask'
        prompt = {sprintf('Doing for masked area\n\nKeep objects larger than, px:'),...
                    'Keep objects smaller than, px:','Current layer only [1-current, 0-all]:',sprintf('3d (Put 0 for 2d mode;\n6, 18, or 26 connection options for 3d)')};
end
answer = inputdlg(prompt,title,[1 30],{'','','1','0'},'on');    
if size(answer) == 0; return; end; 

lo_lim = str2double(answer{1});
hi_lim = str2double(answer{2});
single_layer = str2double(answer{3});
par_3d = str2double(answer{4});
if single_layer
    start_no = 1;
    end_no = 1;
    switch3d = 0;
else
    start_no = 1;
    end_no = size(handles.Img{handles.Id}.I.img, handles.Img{handles.Id}.I.orientation);
    switch3d = 1;
end

wb = waitbar(0,'Please wait...','Name','Size filtration...','WindowStyle','modal');
switch type
    case 'selection'
        ib_do_backup(handles, 'selection', switch3d);
        options.fillBg = 0;
        if par_3d   % do Selection filter in 3D
            selection = ib_getDataset('selection', handles, 4, NaN, options);
            for roi=1:numel(selection)
                if ~isnan(hi_lim)
                    selection{roi} = selection{roi} - uint8(bwareaopen(selection{roi}, hi_lim, par_3d));  % remove objects larger than hi_lim
                end
                if ~isnan(lo_lim)
                    selection{roi} = uint8(bwareaopen(selection{roi}, lo_lim, par_3d));  % remove all 3d objects smaller than lo_lim
                end
            end
            ib_setDataset('selection', selection, handles, 4);
        else  % do Selection filter in 2D
            if start_no==end_no     % a single slice
                selection = ib_getSlice('selection', handles, NaN, NaN, NaN, options);
            else
                selection = ib_getDataset('selection', handles, NaN, NaN, options);
            end
            for roi=1:numel(selection)
                for ind = start_no:end_no
                    waitbar(ind/(end_no-start_no),wb);
                    if ~isnan(hi_lim)
                        selection{roi}(:,:,ind) = selection{roi}(:,:,ind) - uint8(bwareaopen(selection{roi}(:,:,ind), hi_lim));
                    elseif ~isnan(lo_lim)
                        selection{roi}(:,:,ind) = uint8(bwareaopen(selection{roi}(:,:,ind), lo_lim));
                    end
                end
            end
            if start_no==end_no     % a single slice
                ib_setSlice('selection', selection, handles);
            else
                ib_setDataset('selection', selection, handles, NaN);
            end
        end
    case 'model'
        % ib_do_backup(handles, 'model', switch3d);
    case 'mask'
        ib_do_backup(handles, 'mask', switch3d);
        options.fillBg = 0;
        if par_3d   % do Selection filter in 3D
            mask = ib_getDataset('mask', handles, 4, NaN, options);
            for roi=1:numel(mask)
                if ~isnan(hi_lim)
                    mask{roi} = mask{roi} - uint8(bwareaopen(mask{roi}, hi_lim, par_3d));  % remove objects larger than hi_lim
                end
                if ~isnan(lo_lim)
                    mask{roi} = uint8(bwareaopen(mask{roi}, lo_lim, par_3d));  % remove all 3d objects smaller than lo_lim
                end
            end
            ib_setDataset('mask', mask, handles, 4);
        else  % do Selection filter in 2D
            if start_no==end_no     % a single slice
                mask = ib_getSlice('mask', handles, NaN, NaN, NaN, options);
            else
                mask = ib_getDataset('mask', handles, NaN, NaN, options);
            end
            for roi=1:numel(mask)
                for ind = start_no:end_no
                    waitbar(ind/(end_no-start_no),wb);
                    if ~isnan(hi_lim)
                        mask{roi}(:,:,ind) = mask{roi}(:,:,ind) - uint8(bwareaopen(mask{roi}(:,:,ind), hi_lim));
                    elseif ~isnan(lo_lim)
                        mask{roi}(:,:,ind) = uint8(bwareaopen(mask{roi}(:,:,ind), lo_lim));
                    end
                end
            end
            if start_no==end_no     % a single slice
                ib_setSlice('mask', mask, handles);
            else
                ib_setDataset('mask', mask, handles, NaN);
            end
        end
end
delete(wb);
end