function handles = ib_transposeDataset(handles, mode)
% function handles = ib_transposeDataset(handles, mode)
% Transpose dataset physically between dimensions
%
% Parameters:
% handles: handles structure of im_browser
% mode: -> a string that defines the transpose dimension
%     - ''xy2zx'' -> transpose so that XY dimension becomes ZX
%     - ''xy2zy'' -> transpose so that XY dimension becomes ZY
%     - ''zx2zy'' -> transpose so that ZX dimension becomes ZY
%
% Return values:
% handles: handles structure of im_browser

% Copyright (C) 04.07.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 07.09.2015, IB, updated to use imageData.getData4D methods

options.blockModeSwitch = 0;    % overwrite blockmode switch
wb = waitbar(0,sprintf('Transposing the image\nPlease wait...'),'Name',sprintf('Transpose dataset [%s]', mode),'WindowStyle','modal');
tic
handles.U.clearContents(); % clear undo history

% transpose the image layer
img = handles.Img{handles.Id}.I.getData4D('image', NaN, 4, 0, options);   % get dataset (image)
img = transposeme(img, mode);
handles.Img{handles.Id}.I.setData4D('image', img, NaN, 4, 0, options);   % set dataset (image) back
clear img;

% transpose other layers
if strcmp(handles.Img{handles.Id}.I.model_type, 'uint6')
    waitbar(0.5, wb, sprintf('Transposing other layers\nPlease wait...'));
    img = handles.Img{handles.Id}.I.getData4D('everything', NaN, 4, NaN, options);   % get dataset (everything)
    img = transposeme(img, mode);
    handles.Img{handles.Id}.I.setData4D('everything', img, NaN, 4, NaN, options);   % set dataset (everything) back
else
    % transpose selection layer
    waitbar(0.25, wb, sprintf('Transposing the selection layer\nPlease wait...'));
    img = handles.Img{handles.Id}.I.getData4D('selection', NaN, 4, NaN, options);   % get dataset (selection)
    img = transposeme(img, mode);
    handles.Img{handles.Id}.I.setData4D('selection', img, NaN, 4, NaN, options);   % set dataset (selection) back
    
    % transpose mask
    if handles.Img{handles.Id}.I.maskExist
        waitbar(0.5, wb, sprintf('Transposing the mask layer\nPlease wait...'));
        img = handles.Img{handles.Id}.I.getData4D('mask', NaN, 4, NaN, options);   % get dataset (mask)
        img = transposeme(img, mode);
        handles.Img{handles.Id}.I.setData4D('mask', img, NaN, 4, NaN, options);   % set dataset (mask) back
    end
    
    % transpose model
    if handles.Img{handles.Id}.I.modelExist
        waitbar(0.75, wb, sprintf('Transposing the model layer\nPlease wait...'));
        img = handles.Img{handles.Id}.I.getData4D('model', NaN, 4, NaN, options);   % get dataset (model)
        img = transposeme(img, mode);
        handles.Img{handles.Id}.I.setData4D('model', img, NaN, 4, NaN, options);   % set dataset (model) back
    end
end
waitbar(1, wb, sprintf('Finishing...'));

% % update the bounding box
bb = handles.Img{handles.Id}.I.getBoundingBox();    % get current bounding box

switch mode
    case 'xy2zx'
        % swap pixSize.y and pixSize.z
        dummy = handles.Img{handles.Id}.I.pixSize.y;
        handles.Img{handles.Id}.I.pixSize.y = handles.Img{handles.Id}.I.pixSize.z;
        handles.Img{handles.Id}.I.pixSize.z = dummy;
        % update the bounding box
        dummy = bb(3:4);
        bb(3:4) = bb(5:6);
        bb(5:6) = dummy;
    case 'xy2zy'
        % swap pixSize.x and pixSize.z
        dummy = handles.Img{handles.Id}.I.pixSize.x;
        handles.Img{handles.Id}.I.pixSize.x = handles.Img{handles.Id}.I.pixSize.z;
        handles.Img{handles.Id}.I.pixSize.z = dummy;
        % update the bounding box
        dummy = bb(1:2);
        bb(1:2) = bb(5:6);
        bb(5:6) = dummy;
    case 'zx2zy'
        % swap pixSize.y and pixSize.x
        dummy = handles.Img{handles.Id}.I.pixSize.x;
        handles.Img{handles.Id}.I.pixSize.x = handles.Img{handles.Id}.I.pixSize.y;
        handles.Img{handles.Id}.I.pixSize.y = dummy;
        % update the bounding box
        dummy = bb(1:2);
        bb(1:2) = bb(3:4);
        bb(3:4) = dummy;
end
handles.Img{handles.Id}.I.updateBoundingBox(bb);  % update bounding box

log_text = ['Transpose: mode=' mode];
handles.Img{handles.Id}.I.updateImgInfo(log_text);

delete(wb);
toc;
end

function img = transposeme(img, mode)
% transpose function
%     - ''xy2zx'' -> transpose so that XY dimension becomes ZX
%     - ''xy2zy'' -> transpose so that XY dimension becomes ZY
%     - ''zx2zy'' -> transpose so that ZX dimension becomes ZY

if ndims(img) == 4  % for image
    switch mode
        case 'xy2zx'
            img = permute(img,[4, 2, 3, 1]);
        case 'xy2zy'
            img = permute(img,[1, 4, 3, 2]);
        case 'zx2zy'
            img = permute(img,[2, 1, 3, 4]);
    end
else     % for other layers
    switch mode
        case 'xy2zx'
            img = permute(img,[3, 2, 1]);
        case 'xy2zy'
            img = permute(img,[1, 3, 2]);
        case 'zx2zy'
            img = permute(img,[2, 1, 3]);
    end
end
end